@@ -51,6 +51,7 @@ module Control.Retry
     , recovering
     , stepping
     , recoverAll
+    , skipAsyncExceptions
     , logRetries
     , defaultLogMsg
 
@@ -93,7 +94,11 @@ import           GHC.Generics
 import           GHC.Prim
 import           GHC.Types (Int(I#))
 import           System.Random
+# if MIN_VERSION_base(4, 9, 0)
+import           Data.Semigroup
+# else
 import           Data.Monoid
+# endif
 import           Prelude                hiding (catch)
 -------------------------------------------------------------------------------
 
@@ -148,12 +153,28 @@ instance Monad m => Default (RetryPolicyM m) where
     def = constantDelay 50000 <> limitRetries 5
 
 
+-- Base 4.9.0 adds a Data.Semigroup module. This has fewer
+-- dependencies than the semigroups package, so we're using base's
+-- only if its available.
+# if MIN_VERSION_base(4, 9, 0)
+instance Monad m => Semigroup (RetryPolicyM m) where
+  (RetryPolicyM a) <> (RetryPolicyM b) = RetryPolicyM $ \ n -> runMaybeT $ do
+    a' <- MaybeT $ a n
+    b' <- MaybeT $ b n
+    return $! max a' b'
+
+
+instance Monad m => Monoid (RetryPolicyM m) where
+    mempty = retryPolicy $ const (Just 0)
+    mappend = (<>)
+# else
 instance Monad m => Monoid (RetryPolicyM m) where
     mempty = retryPolicy $ const (Just 0)
     (RetryPolicyM a) `mappend` (RetryPolicyM b) = RetryPolicyM $ \ n -> runMaybeT $ do
       a' <- MaybeT $ a n
       b' <- MaybeT $ b n
       return $! max a' b'
+#endif
 
 
 -------------------------------------------------------------------------------
@@ -294,7 +315,7 @@ exponentialBackoff base = retryPolicy $ \ RetryStatus { rsIterNumber = n } ->
 --
 -- temp = min(cap, base * 2 ** attempt)
 --
--- sleep = temp / 2 + random_between(0, temp / 2)
+-- sleep = temp \/ 2 + random_between(0, temp \/ 2)
 fullJitterBackoff :: MonadIO m => Int -> RetryPolicyM m
 fullJitterBackoff base = RetryPolicyM $ \ RetryStatus { rsIterNumber = n } -> do
   let d = (base `boundedMult` boundedPow 2 n) `div` 2
@@ -408,16 +429,29 @@ recoverAll
          -> m a
 recoverAll set f = recovering set handlers f
     where
-#if MIN_VERSION_base(4, 7, 0)
-      someAsyncH _ = Handler $ \(_ :: SomeAsyncException) -> return False
-      handlers = [asyncH, someAsyncH, h]
-#else
-      handlers = [asyncH, h]
-#endif
-      asyncH _ = Handler $ \ (_ :: AsyncException) -> return False
+      handlers = skipAsyncExceptions ++ [h]
       h _ = Handler $ \ (_ :: SomeException) -> return True
 
 
+-------------------------------------------------------------------------------
+-- | List of pre-made handlers that will skip retries on
+-- 'AsyncException' and 'SomeAsyncException'. Append your handlers to
+-- this list as a convenient way to make sure you're not catching
+-- async exceptions like user interrupt.
+skipAsyncExceptions 
+    :: (MonadIO m, MonadMask m) 
+    => [RetryStatus -> Handler m Bool]
+skipAsyncExceptions = handlers
+  where
+    asyncH _ = Handler $ \ (_ :: AsyncException) -> return False
+#if MIN_VERSION_base(4, 7, 0)
+    someAsyncH _ = Handler $ \(_ :: SomeAsyncException) -> return False
+    handlers = [asyncH, someAsyncH]
+#else
+    handlers = [asyncH]
+#endif
+
+
 -------------------------------------------------------------------------------
 -- | Run an action and recover from a raised exception by potentially
 -- retrying the action a number of times. Note that if you're going to
